from netbox.filtersets import NetBoxModelFilterSet
from django.db.models import Q
import django_filters
from . import models

# ThreatSource Filters


class ThreatSourceFilterSet(NetBoxModelFilterSet):
    class Meta:
        model = models.ThreatSource
        fields = ["threat_type", "capability", "intent", "targeting"]


# ThreatEvent Filters


class ThreatEventFilterSet(NetBoxModelFilterSet):
    class Meta:
        model = models.ThreatEvent
        fields = ["threat_source", "relevance", "likelihood", "impact"]


# Vulnerability Filters


class VulnerabilityFilterSet(NetBoxModelFilterSet):

    class Meta:
        model = models.Vulnerability
        fields = ("name", "cve",)

    def search(self, queryset, name, value):
        if not value.strip():
            return queryset
        qs_filter = (
            Q(name__icontains=value)
            | Q(cve__icontains=value)
            | Q(cvssaccessVector__icontains=value)
        )
        return queryset.filter(qs_filter)


# VulnerabilityAssignment Filters


class VulnerabilityAssignmentFilterSet(NetBoxModelFilterSet):
    class Meta:
        model = models.VulnerabilityAssignment
        fields = ["vulnerability"]


# Risk Filters


class RiskFilterSet(NetBoxModelFilterSet):
    class Meta:
        model = models.Risk
        fields = ["name", "threat_event", "description", "impact", "likelihood"]

# Control Filters

class ControlFilterSet(NetBoxModelFilterSet):
    class Meta:
        model = models.Control
        fields = [
            "name",
            "description",
            "notes",
            "category",
            "risk",
        ]


from extras.plugins import PluginConfig
from .version import __version__


class NbriskConfig(PluginConfig):
    name = "nb_risk"
    base_url = "nb_risk"
    verbose_name = "Risk Management"
    description = "NIST 800-30 Risk Management for Netbox"
    version = __version__
    author = "Renato Almdida Oliveira"
    author_email = "renato.almeida.oliveira@gmail.com"
    required_settings = []
    default_settings = {
        "supported_assets": [
            "dcim.device",
            "virtualization.virtualmachine",
            "tenancy.tenant",
            "dcim.site",
        ],
        "additional_assets": [],
    }


config = NbriskConfig  # noqa



# Generated by Django 4.1.5 on 2023-01-25 19:11

from django.db import migrations, models
import django.db.models.deletion
import taggit.managers
import utilities.json


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('contenttypes', '0002_remove_content_type_name'),
        ('extras', '0084_staging'),
    ]

    operations = [
        migrations.CreateModel(
            name='Vulnerability',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False)),
                ('created', models.DateTimeField(auto_now_add=True, null=True)),
                ('last_updated', models.DateTimeField(auto_now=True, null=True)),
                ('custom_field_data', models.JSONField(blank=True, default=dict, encoder=utilities.json.CustomFieldJSONEncoder)),
                ('name', models.CharField(max_length=100, unique=True)),
                ('cve', models.CharField(blank=True, max_length=100)),
                ('description', models.CharField(blank=True, max_length=100)),
                ('notes', models.TextField(blank=True)),
                ('tags', taggit.managers.TaggableManager(through='extras.TaggedItem', to='extras.Tag')),
            ],
            options={
                'verbose_name_plural': 'Vulnerabilities',
            },
        ),
        migrations.CreateModel(
            name='VulnerabilityAssignment',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False)),
                ('created', models.DateTimeField(auto_now_add=True, null=True)),
                ('last_updated', models.DateTimeField(auto_now=True, null=True)),
                ('custom_field_data', models.JSONField(blank=True, default=dict, encoder=utilities.json.CustomFieldJSONEncoder)),
                ('asset_id', models.PositiveIntegerField(blank=True, null=True)),
                ('asset_object_type', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='+', to='contenttypes.contenttype')),
                ('tags', taggit.managers.TaggableManager(through='extras.TaggedItem', to='extras.Tag')),
                ('vulnerability', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, related_name='vulnerability_assignments', to='nb_risk.vulnerability')),
            ],
        ),
        migrations.CreateModel(
            name='ThreatSource',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False)),
                ('created', models.DateTimeField(auto_now_add=True, null=True)),
                ('last_updated', models.DateTimeField(auto_now=True, null=True)),
                ('custom_field_data', models.JSONField(blank=True, default=dict, encoder=utilities.json.CustomFieldJSONEncoder)),
                ('name', models.CharField(max_length=100, unique=True)),
                ('threat_type', models.CharField(default='ADVERSARIAL', max_length=100)),
                ('capability', models.CharField(default='Very High', max_length=100)),
                ('intent', models.CharField(blank=True, max_length=100)),
                ('targeting', models.CharField(blank=True, max_length=100)),
                ('description', models.CharField(blank=True, max_length=100)),
                ('notes', models.TextField(blank=True)),
                ('tags', taggit.managers.TaggableManager(through='extras.TaggedItem', to='extras.Tag')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ThreatEvent',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False)),
                ('created', models.DateTimeField(auto_now_add=True, null=True)),
                ('last_updated', models.DateTimeField(auto_now=True, null=True)),
                ('custom_field_data', models.JSONField(blank=True, default=dict, encoder=utilities.json.CustomFieldJSONEncoder)),
                ('name', models.CharField(max_length=100, unique=True)),
                ('description', models.CharField(blank=True, max_length=100)),
                ('notes', models.TextField(blank=True)),
                ('relevance', models.CharField(default='N/A', max_length=100)),
                ('likelihood', models.CharField(default='Very High', max_length=100)),
                ('impact', models.CharField(max_length=100, unique=True)),
                ('tags', taggit.managers.TaggableManager(through='extras.TaggedItem', to='extras.Tag')),
                ('threat_source', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='threat_events', to='nb_risk.threatsource')),
                ('vulnerability', models.ManyToManyField(blank=True, related_name='threat_events', to='nb_risk.vulnerabilityassignment')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='Risk',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False)),
                ('created', models.DateTimeField(auto_now_add=True, null=True)),
                ('last_updated', models.DateTimeField(auto_now=True, null=True)),
                ('custom_field_data', models.JSONField(blank=True, default=dict, encoder=utilities.json.CustomFieldJSONEncoder)),
                ('name', models.CharField(max_length=100, unique=True)),
                ('notes', models.TextField(blank=True)),
                ('description', models.CharField(blank=True, max_length=100)),
                ('likelihood', models.CharField(default='Very High', max_length=100)),
                ('impact', models.CharField(default='Very High', max_length=100)),
                ('tags', taggit.managers.TaggableManager(through='extras.TaggedItem', to='extras.Tag')),
                ('threat_event', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, related_name='risks', to='nb_risk.threatevent')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.AddConstraint(
            model_name='vulnerabilityassignment',
            constraint=models.UniqueConstraint(fields=('asset_object_type', 'asset_id', 'vulnerability'), name='nb_risk_vulnerabilityassignment_unique_object_vulnerability'),
        ),
    ]



from rest_framework import serializers

from netbox.api.serializers import WritableNestedSerializer
from .. import models, choices

# ThreatSource Nested Serializers

class NestedThreatSourceSerializer(WritableNestedSerializer):
    url = serializers.HyperlinkedIdentityField(
        view_name="plugins-api:nb_risk-api:threatsource-detail"
    )
    display = serializers.SerializerMethodField('get_display')
    
    def get_display(self, obj):
        return obj.name

    class Meta:
        model = models.ThreatSource
        fields = ["id", "url", "display", "name"]
    
# Risk Nested Serializers

class NestedRiskSerializer(WritableNestedSerializer):
    url = serializers.HyperlinkedIdentityField(
        view_name="plugins-api:nb_risk-api:risk-detail"
    )
    display = serializers.SerializerMethodField('get_display')
    
    def get_display(self, obj):
        return obj.name

    class Meta:
        model = models.Risk
        fields = ["id", "url", "display", "name"]


from netbox.api.viewsets import NetBoxModelViewSet

from .. import models
from . import serializers

# ThreatSource ViewSets

class ThreatSourceViewSet(NetBoxModelViewSet):
    queryset = models.ThreatSource.objects.all()
    serializer_class = serializers.ThreatSourceSerializer
    
# ThreatEvent ViewSets

class ThreatEventViewSet(NetBoxModelViewSet):
    queryset = models.ThreatEvent.objects.all()
    serializer_class = serializers.ThreatEventSerializer

# Vulnerability ViewSets

class VulnerabilityViewSet(NetBoxModelViewSet):
    queryset = models.Vulnerability.objects.all()
    serializer_class = serializers.VulnerabilitySerializer
    
# VulnerabilityAssignment ViewSets

class VulnerabilityAssignmentViewSet(NetBoxModelViewSet):
    queryset = models.VulnerabilityAssignment.objects.all()
    serializer_class = serializers.VulnerabilityAssignmentSerializer

# Risk ViewSets

class RiskViewSet(NetBoxModelViewSet):
    queryset = models.Risk.objects.all()
    serializer_class = serializers.RiskSerializer

# Control ViewSets

class ControlViewSet(NetBoxModelViewSet):
    queryset = models.Control.objects.all()
    serializer_class = serializers.ControlSerializer


from django.db import models
from django.contrib.contenttypes.models import ContentType
from django.contrib.contenttypes.fields import GenericForeignKey
from django.urls import reverse

from netbox.models import NetBoxModel
from . import choices

# ThreatSource Model


class ThreatSource(NetBoxModel):

    name = models.CharField("Name", max_length=100, unique=True)
    threat_type = models.CharField(
        "Threat Type",
        max_length=100,
        choices=choices.ThreatTypeChoices,
        default=choices.ThreatTypeChoices.THREAT_TYPE_1,
    )
    capability = models.CharField(
        "Capability",
        max_length=100,
        choices=choices.CapabilityChoices,
        default=choices.CapabilityChoices.CAPABILITY_1,
    )
    intent = models.CharField("Intent", max_length=100, blank=True)
    targeting = models.CharField("Target", max_length=100, blank=True)
    description = models.CharField("Description", max_length=100, blank=True)
    notes = models.TextField("Notes", blank=True)

    def __str__(self):
        return self.name

    def get_absolute_url(self):
        return reverse("plugins:nb_risk:threatsource", args=[self.pk])


# Vulnerability Model


class Vulnerability(NetBoxModel):

    name = models.CharField("Name", max_length=100, unique=True)
    cve = models.CharField("CVE", max_length=100, blank=True)
    description = models.CharField("Description", max_length=100, blank=True)
    notes = models.TextField("Notes", blank=True)
    cvssaccessVector = models.CharField(
        "Access Vector (AV)", max_length=100, blank=True
    )
    cvssaccessComplexity = models.CharField(
        "Access Complexity (AC)", max_length=100, blank=True
    )
    cvssauthentication = models.CharField(
        "Authentication (Au)", max_length=100, blank=True
    )
    cvssconfidentialityImpact = models.CharField(
        "Confidentiality Impact (C)", max_length=100, blank=True
    )
    cvssintegrityImpact = models.CharField(
        "Integrity Impact (I)", max_length=100, blank=True
    )
    cvssavailabilityImpact = models.CharField(
        "Availability Impact (A)", max_length=100, blank=True
    )
    cvssbaseScore = models.FloatField("Base Score", max_length=100, blank=True)

    @property
    def affected_assets(self):
        return self.vulnerability_assignments.count()

    def __str__(self):
        return self.name

    def get_absolute_url(self):
        return reverse("plugins:nb_risk:vulnerability", args=[self.pk])

    class Meta:
        verbose_name = "Vulnerability"
        verbose_name_plural = "Vulnerabilities"


# VulnearbilityAssingment Model


class VulnerabilityAssignment(NetBoxModel):

    asset_object_type = models.ForeignKey(
        to=ContentType,
        on_delete=models.CASCADE,
        related_name="+",
        blank=True,
        null=True,
    )
    asset_id = models.PositiveIntegerField(blank=True, null=True)
    asset = GenericForeignKey(
        ct_field="asset_object_type",
        fk_field="asset_id",
    )
    vulnerability = models.ForeignKey(
        to=Vulnerability,
        on_delete=models.PROTECT,
        related_name="vulnerability_assignments",
        verbose_name="Vulnerability",
    )

    @property
    def name(self):
        return f"{self.asset} - {self.vulnerability.name}"

    def __str__(self):
        return f"{self.asset} - {self.vulnerability.name}"

    class Meta:
        constraints = (
            models.UniqueConstraint(
                fields=("asset_object_type", "asset_id", "vulnerability"),
                name="%(app_label)s_%(class)s_unique_object_vulnerability",
            ),
        )


# ThreatEvent Model


class ThreatEvent(NetBoxModel):

    name = models.CharField("Name", max_length=100, unique=True)
    threat_source = models.ForeignKey(
        to=ThreatSource,
        on_delete=models.PROTECT,
        related_name="threat_events",
        verbose_name="Threat Source",
        blank=True,
        null=True,
    )
    description = models.CharField("Description", max_length=100, blank=True)
    notes = models.TextField("Notes", blank=True)
    relevance = models.CharField(
        "Relevance",
        max_length=100,
        choices=choices.RelevanceChoices,
        default=choices.RelevanceChoices.RELEVANCE_6,
    )
    likelihood = models.CharField(
        "Likelihood",
        max_length=100,
        choices=choices.LikelihoodChoices,
        default=choices.LikelihoodChoices.LIKELIHOOD_1,
    )
    impact = models.CharField("Impact", max_length=100, unique=True)

    vulnerability = models.ManyToManyField(
        to=VulnerabilityAssignment,
        related_name="threat_events",
        blank=True,
    )

    def __str__(self):
        return self.name

    def get_absolute_url(self):
        return reverse("plugins:nb_risk:threatevent", args=[self.pk])


# Risk Model


class Risk(NetBoxModel):

    name = models.CharField("Name", max_length=100, unique=True)
    notes = models.TextField("Notes", blank=True)
    threat_event = models.ForeignKey(
        to=ThreatEvent,
        on_delete=models.PROTECT,
        related_name="risks",
        verbose_name="Threat Event",
    )
    description = models.CharField("Description", max_length=100, blank=True)
    likelihood = models.CharField(
        "Likelihood",
        max_length=100,
        choices=choices.LikelihoodChoices,
        default=choices.LikelihoodChoices.LIKELIHOOD_1,
    )
    impact = models.CharField(
        "Level of Impact",
        max_length=100,
        choices=choices.ImpactChoices,
        default=choices.ImpactChoices.IMPACT_1,
    )

    @property
    def risk_level(self):
        if self.impact == choices.ImpactChoices.IMPACT_5:
            return choices.RiskLevelChoices.RISK_LEVEL_5
        elif self.impact == choices.ImpactChoices.IMPACT_4:
            if self.likelihood == choices.LikelihoodChoices.LIKELIHOOD_5:
                return choices.RiskLevelChoices.RISK_LEVEL_5
            else:
                return choices.RiskLevelChoices.RISK_LEVEL_4
        elif self.impact == choices.ImpactChoices.IMPACT_3:
            if self.likelihood == choices.LikelihoodChoices.LIKELIHOOD_5:
                return choices.RiskLevelChoices.RISK_LEVEL_5
            elif self.likelihood == choices.LikelihoodChoices.LIKELIHOOD_4:
                return choices.RiskLevelChoices.RISK_LEVEL_4
            else:
                return choices.RiskLevelChoices.RISK_LEVEL_3
        elif self.impact == choices.ImpactChoices.IMPACT_2:
            if self.likelihood == choices.LikelihoodChoices.LIKELIHOOD_5:
                return choices.RiskLevelChoices.RISK_LEVEL_4
            elif self.likelihood == choices.LikelihoodChoices.LIKELIHOOD_4:
                return choices.RiskLevelChoices.RISK_LEVEL_4
            elif self.likelihood == choices.LikelihoodChoices.LIKELIHOOD_3:
                return choices.RiskLevelChoices.RISK_LEVEL_3
            else:
                return choices.RiskLevelChoices.RISK_LEVEL_2
        elif self.impact == choices.ImpactChoices.IMPACT_1:
            if self.likelihood == choices.LikelihoodChoices.LIKELIHOOD_5:
                return choices.RiskLevelChoices.RISK_LEVEL_4
            elif self.likelihood == choices.LikelihoodChoices.LIKELIHOOD_4:
                return choices.RiskLevelChoices.RISK_LEVEL_3
            elif self.likelihood == choices.LikelihoodChoices.LIKELIHOOD_3:
                return choices.RiskLevelChoices.RISK_LEVEL_2
            elif self.likelihood == choices.LikelihoodChoices.LIKELIHOOD_2:
                return choices.RiskLevelChoices.RISK_LEVEL_1
            else:
                return choices.RiskLevelChoices.RISK_LEVEL_1

    def __str__(self):
        return self.name

    def get_absolute_url(self):
        return reverse("plugins:nb_risk:risk", args=[self.pk])


# Control Model

class Control(NetBoxModel):

    name = name = models.CharField("Name", max_length=100, unique=True)
    description = models.CharField("Description", max_length=100, blank=True)
    notes = models.TextField("Notes", blank=True)
    category = models.CharField(
        "Category",
        max_length=100,
        choices=choices.ControlCategoryChoices,
        default=choices.ControlCategoryChoices.CATEGORY_1,
    )
    risk = models.ManyToManyField(
        Risk,
        verbose_name="Risks",
        blank=True,
    )

    def __str__(self):
        return self.name

    def get_absolute_url(self):
        return reverse("plugins:nb_risk:control", args=[self.pk])



from django.contrib import admin
from . import models


@admin.register(models.VulnerabilityAssignment)
class VulnerabilityAssignmentAdmin(admin.ModelAdmin):
    fields = ["asset_object_type", "asset_id", "vulnerability"]



import django_tables2 as tables
from django.urls import reverse
from django.utils.safestring import mark_safe
from django.utils.http import urlencode


class CreateColumn(tables.Column):
    attrs = {"td": {"class": "text-end text-nowrap"}}

    def render(self, record):
        cve = record["id"]
        description = record["description"] 
        if "accessVector" in record:
            accessVector = record["accessVector"]
        else:
            accessVector = ""
        if "accessComplexity" in record:
            accessComplexity = record["accessComplexity"]
        else:
            accessComplexity = ""
        if "authentication" in record:
            authentication = record["authentication"]
        else:
            authentication = ""
        if "confidentialityImpact" in record:
            confidentialityImpact = record["confidentialityImpact"]
        else:
            confidentialityImpact = ""
        if "integrityImpact" in record:
            integrityImpact = record["integrityImpact"]
        else:
            integrityImpact = ""
        if "availabilityImpact" in record:
            availabilityImpact = record["availabilityImpact"]
        else:
            availabilityImpact = ""
        if "baseScore" in record:
            baseScore = record["baseScore"]
        else:
            baseScore = ""


        url = reverse("plugins:nb_risk:vulnerability_add")
        query = {
            "cve": cve,
            "notes": description,
            "name": cve,
            "cvssaccessVector": accessVector,
            "cvssaccessComplexity": accessComplexity,
            "cvssauthentication": authentication,
            "cvssconfidentialityImpact": confidentialityImpact,
            "cvssintegrityImpact": integrityImpact,
            "cvssavailabilityImpact": availabilityImpact,
            "cvssbaseScore": baseScore,
        }
        encoded_query = urlencode(query)
        url = f"{url}?{encoded_query}"

        html = f'<a href={url} class="btn btn-primary btn-sm"><i class="mdi mdi-plus" aria-hidden="true"></i></a>'
        return mark_safe(html)



from extras.plugins import PluginMenu, PluginMenuItem, PluginMenuButton
from utilities.choices import ButtonColorChoices

menu = PluginMenu(
    label="Risk Assessment",
    icon_class="mdi mdi-spider",
    groups=(
        (
            "Threat",
            (
                PluginMenuItem(
                    permissions=["nb_risk.view_threatsource"],
                    link="plugins:nb_risk:threatsource_list",
                    link_text="Threat Sources",
                    buttons=(
                        PluginMenuButton(
                            "plugins:nb_risk:threatsource_add",
                            "Add",
                            "mdi mdi-plus-thick",
                            ButtonColorChoices.GREEN,
                            permissions=["nb_risk.add_threatsource"],
                        ),
                    ),
                ),
                PluginMenuItem(
                    permissions=["nb_risk.view_threatevent"],
                    link="plugins:nb_risk:threatevent_list",
                    link_text="Threat Events",
                    buttons=(
                        PluginMenuButton(
                            "plugins:nb_risk:threatevent_add",
                            "Add",
                            "mdi mdi-plus-thick",
                            ButtonColorChoices.GREEN,
                            permissions=["nb_risk.add_threatevent"],
                        ),
                    ),
                ),
            ),
        ),
        (
            "Vulnerability",
            (
                PluginMenuItem(
                    permissions=["nb_risk.view_vulnerability"],
                    link="plugins:nb_risk:vulnerability_list",
                    link_text="Vulnerabilities",
                    buttons=(
                        PluginMenuButton(
                            "plugins:nb_risk:vulnerability_add",
                            "Add",
                            "mdi mdi-plus-thick",
                            ButtonColorChoices.GREEN,
                            permissions=["nb_risk.add_vulnerability"],
                        ),
                        PluginMenuButton(
                            "plugins:nb_risk:vulnerability_search",
                            "Search",
                            "mdi mdi-magnify",
                            ButtonColorChoices.BLUE,
                            permissions=["nb_risk.view_vulnerability"],
                        ),
                    ),
                ),
                PluginMenuItem(
                    permissions=["nb_risk.view_vulnerabilityassignment"],
                    link="plugins:nb_risk:vulnerabilityassignment_list",
                    link_text="Vulnerability Assignments",
                ),
            ),
        ),
        (
            "Risk",
            (
                PluginMenuItem(
                    permissions=["nb_risk.view_risk"],
                    link="plugins:nb_risk:risk_list",
                    link_text="Risks",
                    buttons=(
                        PluginMenuButton(
                            "plugins:nb_risk:risk_add",
                            "Add",
                            "mdi mdi-plus-thick",
                            ButtonColorChoices.GREEN,
                            permissions=["nb_risk.add_risk"],
                        ),
                    ),
                ),
                PluginMenuItem(
                    permissions=["nb_risk.view_control"],
                    link="plugins:nb_risk:control_list",
                    link_text="Controls",
                    buttons=(
                        PluginMenuButton(
                            "plugins:nb_risk:control_add",
                            "Add",
                            "mdi mdi-plus-thick",
                            ButtonColorChoices.GREEN,
                            permissions=["nb_risk.add_control"],
                        ),
                    ),
                ),
            ),
        ),
    ),
)



from django.urls import path, include
from utilities.urls import get_model_urls
from netbox.views.generic import ObjectChangeLogView, ObjectJournalView

from . import models, views, cve

app_name = 'nb_risk'

urlpatterns = (
    # ThreatSource URLs
    path(
        "threat-source/", views.ThreatSourceListView.as_view(), name="threatsource_list"
    ),
    path(
        "threat-source/add/",
        views.ThreatSourceEditView.as_view(),
        name="threatsource_add",
    ),
    path(
        "threat-source/<int:pk>/", views.ThreatSourceView.as_view(), name="threatsource"
    ),
    path(
        "threat-source/<int:pk>/edit/",
        views.ThreatSourceEditView.as_view(),
        name="threatsource_edit",
    ),
    path(
        "threat-source/<int:pk>/delete/",
        views.ThreatSourceDeleteView.as_view(),
        name="threatsource_delete",
    ),
    path(
        "threat-source/<int:pk>/changelog/",
        ObjectChangeLogView.as_view(),
        name="threatsource_changelog",
        kwargs={"model": models.ThreatSource},
    ),
    # ThreatEvent URLs
    path("threat-event/", views.ThreatEventListView.as_view(), name="threatevent_list"),
    path(
        "threat-event/add/", views.ThreatEventEditView.as_view(), name="threatevent_add"
    ),
    path("threat-event/<int:pk>/", views.ThreatEventView.as_view(), name="threatevent"),
    path('threat-event/<int:pk>/', include(get_model_urls(app_name, 'threatevent'))),
    path(
        "threat-event/<int:pk>/edit/",
        views.ThreatEventEditView.as_view(),
        name="threatevent_edit",
    ),
    path(
        "threat-event/<int:pk>/delete/",
        views.ThreatEventDeleteView.as_view(),
        name="threatevent_delete",
    ),
    path(
        "threat-event/delete/",
        views.ThreatEventBulkDeleteView.as_view(),
        name="threatevent_bulk_delete",
    ),
    # Vulnerability URLs
    path(
        "vulnerability/",
        views.VulnerabilityListView.as_view(),
        name="vulnerability_list",
    ),
    path(
        "vulnerability/add/",
        views.VulnerabilityEditView.as_view(),
        name="vulnerability_add",
    ),
    path(
        "vulnerability/<int:pk>/",
        views.VulnerabilityView.as_view(),
        name="vulnerability",
    ),
    path(
        "vulnerability/<int:pk>/edit/",
        views.VulnerabilityEditView.as_view(),
        name="vulnerability_edit",
    ),
    path('vulnerability/import/', views.VulnerabilityBulkImportView.as_view(), name='vulnerability_import'),
    path(
        "vulnerability/<int:pk>/delete/",
        views.VulnerabilityDeleteView.as_view(),
        name="vulnerability_delete",
    ),
    path(
        "vulnerability/delete/",
        views.VulnerabilityBulkDeleteView.as_view(),
        name="vulnerability_bulk_delete",
    ),
    path('vulnerability/<int:pk>/', include(get_model_urls(app_name, 'vulnerability'))),
    path(
        "vulnerability/search/",
        cve.VulnerabilitySearchView.as_view(),
        name="vulnerability_search",
    ),
    # VulnerabilityAssignment URLs
    path(
        "vulnerability-assignments/add/",
        views.VulnerabilityAssignmentEditView.as_view(),
        name="vulnerabilityassignment_add",
    ),
    path("vulnerability-assignments/",
        views.VulnerabilityAssignmentListView.as_view(),
        name="vulnerabilityassignment_list",
    ),
    path(
        "vulnerability-assignments/<int:pk>/delete/",
        views.VulnerabilityAssignmentDeleteView.as_view(),
        name="vulnerabilityassignment_delete",
    ),
    # Risk URLs
    path("risk/", views.RiskListView.as_view(), name="risk_list"),
    path("risk/add/", views.RiskEditView.as_view(), name="risk_add"),
    path("risk/<int:pk>/", views.RiskView.as_view(), name="risk"),
    path("risk/<int:pk>/edit/", views.RiskEditView.as_view(), name="risk_edit"),
    path("risk/<int:pk>/delete/", views.RiskDeleteView.as_view(), name="risk_delete"),
    path("risk/delete/", views.RiskBulkDeleteView.as_view(), name="risk_bulk_delete"),
    path(
        "risk/<int:pk>/changelog/",
        ObjectChangeLogView.as_view(),
        name="risk_changelog",
        kwargs={"model": models.Risk},
    ),
    path(
        "risk/<int:pk>/journal/",
        ObjectJournalView.as_view(),
        name="risk_journal",
        kwargs={"model": models.Risk},
    ),
    # Control URLs
    path("control/", views.ControlListView.as_view(), name="control_list"),
    path("control/add/", views.ControlEditView.as_view(), name="control_add"),
    path("control/<int:pk>/", views.ControlView.as_view(), name="control"),
    path("control/<int:pk>/edit/", views.ControlEditView.as_view(), name="control_edit"),
    path(
        "control/<int:pk>/delete/", views.ControlDeleteView.as_view(), name="control_delete"
    ),
    path(
        "control/delete/", views.ControlBulkDeleteView.as_view(), name="control_bulk_delete"
    ),
    path('control/<int:pk>/', include(get_model_urls(app_name, 'control'))),
    
)
